@@ -1,5 +1,7 @@
 #include <string.h>
+#include <stdlib.h>
 #include "everything.h"
+#include <gio/gio.h>
 
 /* basic types */
 gboolean test_boolean (gboolean in)
@@ -257,6 +259,61 @@ GSList *test_filename_return (void)
   return filenames;
 }
 
+/* in arguments after out arguments */
+
+/**
+ * test_int_out_utf8:
+ * @out: (out):
+ * @in:
+ */
+void
+test_int_out_utf8 (int *length, const char *in)
+{
+    *length = g_utf8_strlen(in, -1);
+}
+
+
+/* multiple output arguments */
+
+/**
+ * test_multi_double_args:
+ * @in:
+ * @one: (out): 
+ * @two: (out): 
+ */
+void
+test_multi_double_args (gdouble in, gdouble *one, gdouble *two)
+{
+  *one = in * 2;
+  *two = in * 3;
+}
+
+/**
+ * test_utf8_out_out:
+ * @out0: (out) (transfer full): a copy of "first"
+ * @out1: (out) (transfer full): a copy of "second"
+ */
+void
+test_utf8_out_out (char **out0, char **out1)
+{
+  *out0 = g_strdup ("first");
+  *out1 = g_strdup ("second");
+}
+
+/**
+ * test_utf8_out_nonconst_return:
+ * @out: (out) (transfer full): a copy of "second"
+ *
+ * Returns: (transfer full): a copy of "first"
+ */
+char *
+test_utf8_out_nonconst_return (char **out)
+{
+  *out = g_strdup ("second");
+  return g_strdup ("first");
+}
+
+
 /* non-basic-types */
 
 static const char *test_sequence[] = {"1", "2", "3"};
@@ -277,6 +334,41 @@ test_array_int_in (int n_ints, int *ints)
   return sum;
 }
 
+/**
+ * test_array_int_out:
+ * @n_ints: (out): the length of @ints
+ * @ints: (out) (array length=n_ints) (transfer full): a list of 5 integers, from 0 to 4 in consecutive order
+ */
+void
+test_array_int_out (int *n_ints, int **ints)
+{
+  int i;
+  *n_ints = 5;
+  *ints = g_malloc0(sizeof(**ints) * *n_ints);
+  for (i = 1; i < *n_ints; i++)
+    (*ints)[i] = (*ints)[i-1] + 1;
+}
+
+/**
+ * test_array_int_inout:
+ * @n_ints: (inout): the length of @ints
+ * @ints: (inout) (array length=n_ints) (transfer full): a list of integers whose items will be increased by 1, except the first that will be dropped
+ */
+void
+test_array_int_inout (int *n_ints, int **ints)
+{
+  int i;
+
+  for (i = 1; i < *n_ints; i++) {
+	(*ints)[i-1] = (*ints)[i] + 1;
+  }
+
+  if (0 < *n_ints) {
+    *n_ints -= 1;
+  }
+  *ints = g_realloc(*ints, sizeof(**ints) * *n_ints);
+}
+
 /**
  * test_array_gint8_in:
  * @n_ints:
@@ -439,6 +531,48 @@ test_strv_outarg (char ***retp)
   *retp = ret;
 }
 
+/**
+ * test_array_fixed_size_int_in:
+ * @ints: (array fixed-size=5): a list of 5 integers
+ *
+ * Returns: the sum of the items in @ints
+ */
+int
+test_array_fixed_size_int_in (int *ints)
+{
+  int i, sum = 0;
+  for (i = 0; i < 5; i++)
+    sum += ints[i];
+  return sum;
+}
+
+/**
+ * test_array_fixed_size_int_out:
+ * @ints: (out) (array fixed-size=5) (transfer full): a list of 5 integers ranging from 0 to 4
+ */
+void
+test_array_fixed_size_int_out (int **ints)
+{
+  int i;
+  *ints = g_malloc0(sizeof(**ints) * 5);
+  for (i = 1; i < 5; i++)
+    (*ints)[i] = (*ints)[i-1] + 1;
+}
+
+/**
+ * test_array_fixed_size_int_return:
+ * Returns: (array fixed-size=5) (transfer full): a list of 5 integers ranging from 0 to 4
+ */
+int *
+test_array_fixed_size_int_return (void)
+{
+  int i, *ints;
+  ints = g_malloc0(sizeof(*ints) * 5);
+  for (i = 1; i < 5; i++)
+    ints[i] = ints[i-1] + 1;
+  return ints;
+}
+
 /**
  * test_array_int_in_take:
  * @n_ints:
@@ -977,7 +1111,7 @@ test_flags_get_type (void)
         static const GFlagsValue values[] = {
             { TEST_FLAG1, "TEST_FLAG1", "flag1" },
             { TEST_FLAG2, "TEST_FLAG2", "flag2" },
-            { TEST_FLAG3, "TEST_FLAG3", "flag2" },
+            { TEST_FLAG3, "TEST_FLAG3", "flag3" },
             { 0, NULL, NULL }
         };
         etype = g_flags_register_static (g_intern_static_string ("TestFlags"), values);
@@ -1123,6 +1257,39 @@ test_boxed_new (void)
   return boxed;
 }
 
+TestBoxed *
+test_boxed_new_alternative_constructor1 (int i)
+{
+  TestBoxed *boxed = g_slice_new0(TestBoxed);
+  boxed->priv = g_slice_new0(TestBoxedPrivate);
+  boxed->priv->magic = 0xdeadbeef;
+  boxed->some_int8 = i;
+
+  return boxed;
+}
+
+TestBoxed *
+test_boxed_new_alternative_constructor2 (int i, int j)
+{
+  TestBoxed *boxed = g_slice_new0(TestBoxed);
+  boxed->priv = g_slice_new0(TestBoxedPrivate);
+  boxed->priv->magic = 0xdeadbeef;
+  boxed->some_int8 = i + j;
+
+  return boxed;
+}
+
+TestBoxed *
+test_boxed_new_alternative_constructor3 (char *s)
+{
+  TestBoxed *boxed = g_slice_new0(TestBoxed);
+  boxed->priv = g_slice_new0(TestBoxedPrivate);
+  boxed->priv->magic = 0xdeadbeef;
+  boxed->some_int8 = atoi(s);
+
+  return boxed;
+}
+
 TestBoxed *
 test_boxed_copy (TestBoxed *boxed)
 {
@@ -1337,12 +1504,75 @@ test_obj_set_bare (TestObj *obj, GObject *bare)
     g_object_ref (obj->bare);
 }
 
+int
+test_obj_instance_method (TestObj *obj)
+{
+    return -1;
+}
+
 double
 test_obj_static_method (int x)
 {
   return x;
 }
 
+/**
+ * test_obj_torture_signature_0:
+ * @obj: A #TestObj
+ * @x:
+ * @y: (out):
+ * @z: (out):
+ * @foo:
+ * @q: (out):
+ * @m:
+ *
+ */
+void
+test_obj_torture_signature_0 (TestObj    *obj,
+                              int         x,
+                              double     *y,
+                              int        *z,
+                              const char *foo,
+                              int        *q,
+                              guint       m)
+{
+  *y = x;
+  *z = x * 2;
+  *q = g_utf8_strlen (foo, -1) + m;
+}
+
+/**
+ * test_obj_torture_signature_1:
+ * @obj: A #TestObj
+ * @x:
+ * @y: (out):
+ * @z: (out):
+ * @foo:
+ * @q: (out):
+ * @m:
+ * @error: A #GError
+ *
+ * This function throws an error if m is odd.
+ */
+gboolean
+test_obj_torture_signature_1 (TestObj   *obj,
+                              int        x,
+                              double     *y,
+                              int        *z,
+                              const char *foo,
+                              int        *q,
+                              guint       m,
+                              GError    **error)
+{
+  *y = x;
+  *z = x * 2;
+  *q = g_utf8_strlen (foo, -1) + m;
+  if (m % 2 == 0)
+      return TRUE;
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "m is odd");
+  return FALSE;
+}
+
 /**
  * test_obj_do_matrix:
  * @obj: A #TestObj
@@ -1370,15 +1600,48 @@ struct _CallbackInfo
 };
 
 
+G_DEFINE_TYPE(TestSubObj, test_sub_obj, TEST_TYPE_OBJ);
+
+static void
+test_sub_obj_class_init (TestSubObjClass *klass)
+{
+}
+
+static void
+test_sub_obj_init (TestSubObj *obj)
+{
+}
+
+TestSubObj*
+test_sub_obj_new ()
+{
+  return g_object_new (TEST_TYPE_SUB_OBJ, NULL);
+}
+
+int
+test_sub_obj_instance_method (TestSubObj *obj)
+{
+    return 0;
+}
+
+void
+test_sub_obj_unset_bare (TestSubObj *obj)
+{
+  test_obj_set_bare(TEST_OBJECT(obj), NULL);
+}
+
+
 /**
  * test_callback:
- * @callback: (scope call):
+ * @callback: (scope call) (allow-none):
  *
  **/
 int
 test_callback (TestCallback callback)
 {
-  return callback();
+    if (callback != NULL)
+        return callback();
+    return 0;
 }
 
 /**
@@ -1414,7 +1677,7 @@ test_callback_destroy_notify (TestCallbackUserData callback,
 
   retval = callback(user_data);
 
-  info = g_new(CallbackInfo, 1);
+  info = g_slice_new(CallbackInfo);
   info->callback = callback;
   info->notify = notify;
   info->user_data = user_data;
@@ -1441,11 +1704,11 @@ test_callback_thaw_notifications (void)
 
   for (node = notified_callbacks; node != NULL; node = node->next)
     {
-      CallbackInfo *info = (CallbackInfo *)node->data;
+      CallbackInfo *info = node->data;
       retval += info->callback (info->user_data);
       if (info->notify)
         info->notify (info->user_data);
-      g_free (info);
+      g_slice_free (CallbackInfo, info);
     }
 
   g_slist_free (notified_callbacks);
@@ -1454,6 +1717,47 @@ test_callback_thaw_notifications (void)
   return retval;
 }
 
+static GSList *async_callbacks = NULL;
+
+/**
+ * test_callback_async:
+ * @callback: (scope async):
+ *
+ **/
+void
+test_callback_async (TestCallbackUserData callback,
+                     gpointer user_data)
+{
+  CallbackInfo *info;
+
+  info = g_slice_new(CallbackInfo);
+  info->callback = callback;
+  info->user_data = user_data;
+
+  async_callbacks = g_slist_prepend(async_callbacks, info);
+}
+
+/**
+ * test_callback_thaw_async:
+ */
+int
+test_callback_thaw_async (void)
+{
+  int retval = 0;
+  GSList *node;
+
+  for (node = async_callbacks; node != NULL; node = node->next)
+    {
+      CallbackInfo *info = node->data;
+      retval = info->callback (info->user_data);
+      g_slice_free (CallbackInfo, info);
+    }
+
+  g_slist_free (async_callbacks);
+  async_callbacks = NULL;
+  return retval;
+}
+
 /**
  * test_callback_infinite:
  * @callback: (scope infinite):
@@ -1600,3 +1904,87 @@ test_wi_802_1x_static_method (int x)
 {
   return 2*x;
 }
+
+/**
+ * test_torture_signature_0:
+ * @x:
+ * @y: (out):
+ * @z: (out):
+ * @foo:
+ * @q: (out):
+ * @m:
+ *
+ */
+void
+test_torture_signature_0 (int         x,
+                          double     *y,
+                          int        *z,
+                          const char *foo,
+                          int        *q,
+                          guint       m)
+{
+  *y = x;
+  *z = x * 2;
+  *q = g_utf8_strlen (foo, -1) + m;
+}
+
+/**
+ * test_torture_signature_1:
+ * @x:
+ * @y: (out):
+ * @z: (out):
+ * @foo:
+ * @q: (out):
+ * @m:
+ * @error: A #GError
+ *
+ * This function throws an error if m is odd.
+ */
+gboolean
+test_torture_signature_1 (int         x,
+                          double     *y,
+                          int        *z,
+                          const char *foo,
+                          int        *q,
+                          guint       m,
+                          GError    **error)
+{
+  *y = x;
+  *z = x * 2;
+  *q = g_utf8_strlen (foo, -1) + m;
+  if (m % 2 == 0)
+      return TRUE;
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "m is odd");
+  return FALSE;
+}
+
+/**
+ * test_torture_signature_2:
+ * @x:
+ * @callback:
+ * @user_data:
+ * @notify:
+ * @y: (out):
+ * @z: (out):
+ * @foo:
+ * @q: (out):
+ * @m:
+ *
+ */
+void
+test_torture_signature_2 (int                   x,
+                          TestCallbackUserData  callback,
+                          gpointer              user_data,
+                          GDestroyNotify        notify,
+                          double               *y,
+                          int                  *z,
+                          const char           *foo,
+                          int                  *q,
+                          guint                 m)
+{
+  *y = x;
+  *z = x * 2;
+  *q = g_utf8_strlen (foo, -1) + m;  
+  notify (user_data);
+}
+
